/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/
#include <project.h>
#include <stdio.h>

void AlarmHandler();
void StackEventHandler(uint32 eventCode, void *eventParam);
void FlagHandler();
void PrintTime();

CYBLE_GATT_HANDLE_VALUE_PAIR_T      sleepHandle;

/* Time: 12:00:00 */
#define TIME_HOUR               (0x12u)
#define TIME_MIN                (0x00u)
#define TIME_SEC                (0x00u)
#define HOUR_OFFSET             (16u)
#define MIN_OFFSET              (8u)
#define TIME_HR_MIN_SEC         ((uint32)(TIME_HOUR << HOUR_OFFSET) | \
                                (uint32)(TIME_MIN << MIN_OFFSET)    | \
                                TIME_SEC)
/* Date: 03/22/2014 */
#define DATE_MONTH              (RTC_AUGUST)
#define DATE_DAY                (0x03u)
#define DATE_YEAR               (0x2017u)
#define MONTH_OFFSET            (24u)
#define DAY_OFFSET              (16u)
#define DATE_MONTH_DAY_YEAR     ((uint32)(DATE_MONTH << MONTH_OFFSET)   | \
                                (uint32)(DATE_DAY << DAY_OFFSET)        | \
                                DATE_YEAR)
                            
/* Time: 12:01:00 */
#define ALARM_TIME_HOUR         (0x12u)
#define ALARM_TIME_MIN          (0x01u)
#define ALARM_TIME_SEC          (0x00u)
#define HOUR_OFFSET             (16u)
#define MIN_OFFSET              (8u)
#define ALARM_TIME_HR_MIN_SEC   ((uint32)(ALARM_TIME_HOUR << HOUR_OFFSET) | \
                                (uint32)(ALARM_TIME_MIN << MIN_OFFSET)    | \
                                ALARM_TIME_SEC)

uint8 bleOn = 0;
uint8 sleepTimeByte[4];
uint32 sleepTime;
uint8 sleepNow;
uint8 sleeping;
uint32 time = 0;
char output[100];

/* Alarm structure declaration */
RTC_DATE_TIME alarmTimeDate;

CYBLE_BLESS_STATE_T bless;

int main()
{
    /* Place your initialization/startup code here (e.g. MyInst_Start()) */
   
    CyGlobalIntEnable; /* Uncomment this line to enable global interrupts. */
    UART_Start();
    //RTC_Start();
    //RTC_SetDateAndTime (TIME_HR_MIN_SEC, DATE_MONTH_DAY_YEAR);
    RTC_SetAlarmMask (RTC_ALARM_MIN_MASK|RTC_ALARM_HOUR_MASK);
    alarmTimeDate.time = ALARM_TIME_HR_MIN_SEC;
    alarmTimeDate.date = DATE_MONTH_DAY_YEAR;
    /* Set function AlarmIsrHandler to be called when alarm triggers */
    RTC_SetAlarmHandler(AlarmHandler);
    CyBle_Start(StackEventHandler);
    
    /* Register EnableRtcOperation() by the COUNTER0. */
    CySysWdtSetInterruptCallback(CY_SYS_WDT_COUNTER2, PrintTime);
    
    
    
    for(;;)
    {
        if(0 == sleeping)
        {
            CyBle_ProcessEvents();
        }
        FlagHandler();
    }
}



void AlarmHandler()
{
    if(bleOn != 1)
    {
        UART_UartPutString("BLE Start\r\n");
        //CyBle_Start(StackEventHandler);
        CyBle_ExitLPM();  
        CyBle_ProcessEvents();
        CyBle_GappStartAdvertisement(CYBLE_ADVERTISING_FAST);
        CyBle_ProcessEvents();
        bleOn = 1;
        sleeping = 0;
    }
    UART_UartPutString("Alarm\r\n");
    RTC_ClearAlarmStatus();
    RTC_Stop();
}

void StackEventHandler(uint32 eventCode, void *eventParam)
{
    uint8 calibrateFlag;
    CYBLE_GATTS_WRITE_REQ_PARAM_T *wrReqParam;
    
    switch( eventCode )
    {
        /* Generic events */

        case CYBLE_EVT_HOST_INVALID:
        break;

        case CYBLE_EVT_STACK_ON:
            CyBle_GappStartAdvertisement(CYBLE_ADVERTISING_FAST);
            UART_UartPutString("Stack On\r\n");
            
            //CyBle_GapFixAuthPassKey(TRUE, passKey);
        break;

        case CYBLE_EVT_TIMEOUT:
        break;
            
        /* GAP events */

        case CYBLE_EVT_GAP_AUTH_REQ:
            sprintf(output," DEBUG: CYBLE_EVT_GAP_AUTH_REQ: security=0x%x, bonding=0x%x, ekeySize=0x%x, err=0x%x \r\n",
            (*(CYBLE_GAP_AUTH_INFO_T *)eventParam).security,
            (*(CYBLE_GAP_AUTH_INFO_T *)eventParam).bonding,
            (*(CYBLE_GAP_AUTH_INFO_T *)eventParam).ekeySize,
            (*(CYBLE_GAP_AUTH_INFO_T *)eventParam).authErr);
            UART_UartPutString(output);
        break;

        case CYBLE_EVT_GAP_PASSKEY_ENTRY_REQUEST:
        break;

        case CYBLE_EVT_GAP_PASSKEY_DISPLAY_REQUEST:
            UART_UartPutString("\r\n");
            sprintf(output,"CYBLE_EVT_GAP_PASSKEY_DISPLAY_REQUEST. Passkey is: %lu \r\n", *(uint32 *)eventParam);
            UART_UartPutString(output);
            UART_UartPutString("Please enter the passkey on your Server device.\r\n");
        break;

        case CYBLE_EVT_GAP_AUTH_COMPLETE:
        break;

        case CYBLE_EVT_GAP_AUTH_FAILED:
        break;

        case CYBLE_EVT_GAP_DEVICE_CONNECTED:
            UART_UartPutString("GAP Event Connected\r\n");
        break;

        case CYBLE_EVT_GAP_DEVICE_DISCONNECTED:
            UART_UartPutString("GAP Event Disconnected\r\n");
            CyBle_GappStartAdvertisement(CYBLE_ADVERTISING_FAST); 
        break;
            
        case CYBLE_EVT_GATTS_WRITE_REQ:
            UART_UartPutString("Write Request Received\r\n");
            wrReqParam = (CYBLE_GATTS_WRITE_REQ_PARAM_T *) eventParam;
            
            
            if(wrReqParam->handleValPair.attrHandle == cyBle_customs[CYBLE_TIME_SERVICE_INDEX].\
								customServiceInfo[CYBLE_TIME_SLEEPNOW_CHAR_INDEX].customServiceCharHandle)
            {
                sleepNow = *(wrReqParam->handleValPair.value.val);
                sprintf(output,"Sleep Now: %u\r\n",sleepNow);
                UART_UartPutString(output);
                sleepHandle.attrHandle = CYBLE_TIME_SLEEPNOW_CHAR_HANDLE;				
	            sleepHandle.value.val = &sleepNow;
	            sleepHandle.value.len = 1;
                CyBle_GattsWriteAttributeValue(&sleepHandle, 0, &cyBle_connHandle, CYBLE_GATT_DB_LOCALLY_INITIATED);
                CyBle_ProcessEvents();
            }
            
            if(wrReqParam->handleValPair.attrHandle == cyBle_customs[CYBLE_TIME_SERVICE_INDEX].\
								customServiceInfo[CYBLE_TIME_SLEEPTIME_CHAR_INDEX].customServiceCharHandle)
            {
                UART_UartPutString("Sleep Time Write Request. \r\n");
                sleepTimeByte[3] = wrReqParam->handleValPair.value.val[3];
                sleepTimeByte[2] = wrReqParam->handleValPair.value.val[2];
                sleepTimeByte[1] = wrReqParam->handleValPair.value.val[1];
                sleepTimeByte[0] = wrReqParam->handleValPair.value.val[0];
                
                sleepTime = (0x000000FF & sleepTimeByte[3]);
	            sleepTime = sleepTime | ((0x000000FF & sleepTimeByte[2]) << 8);
	            sleepTime = sleepTime | ((0x000000FF & sleepTimeByte[1]) << 16);
	            sleepTime = sleepTime | ((0x000000FF & sleepTimeByte[0]) << 24);
                
                sprintf(output,"Sleep Time: %u:%u:%u:%u\r\n",sleepTimeByte[0],sleepTimeByte[1],sleepTimeByte[2],sleepTimeByte[3]);
                UART_UartPutString(output);
                sleepHandle.attrHandle = CYBLE_TIME_SLEEPTIME_CHAR_HANDLE;				
	            sleepHandle.value.val = sleepTimeByte;
	            sleepHandle.value.len = 4;
                CyBle_GattsWriteAttributeValue(&sleepHandle, 0, &cyBle_connHandle, CYBLE_GATT_DB_LOCALLY_INITIATED);
                CyBle_ProcessEvents();
                //alarmTimeDate.time = sleepTime;
            }
            
        break;
     }
     
}

void FlagHandler()
{
    if(sleepNow == 1)
    {
        UART_UartPutString("Sleep Now. \r\n");
        sleepNow = 0;
        CyBle_ProcessEvents();
        RTC_Start();
        RTC_SetDateAndTime (TIME_HR_MIN_SEC, DATE_MONTH_DAY_YEAR);
        RTC_SetAlarmDateAndTime(&alarmTimeDate);
        sleeping = 1;
        CyBle_EnterLPM(CYBLE_BLESS_DEEPSLEEP);
        bless = CyBle_GetBleSsState();
        if(bless == CYBLE_BLESS_STATE_ECO_ON)
        {
            UART_UartPutString("CYBLE_BLESS_STATE_ECO_ON. \r\n");
            CyBle_GapDisconnect(cyBle_connHandle.bdHandle); 
            CySysPmDeepSleep();
        }
        else if(bless == CYBLE_BLESS_STATE_DEEPSLEEP)
        {
           UART_UartPutString("CYBLE_BLESS_STATE_DEEPSLEEP. \r\n");   
           CySysPmDeepSleep();
        }
    }
    
}

void PrintTime()
{
    time = RTC_GetTime();
    sprintf(output, "Time: %lu\r\n", time);
    //UART_UartPutString(output);
}



/* [] END OF FILE */
